==> ja-WiKicker-0.23.tbz: lib/perl5/site_perl/5.8.5/WiKicker/WWW/Robot/RobotUriQueue.pm <==
#
# WiKicker
#
#  RobotUriQueue.pm
#
#
# Copyright (C) 2003 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: RobotUriQueue.pm,v 1.1 2003/12/18 12:27:58 naney Exp $
#
#

package WiKicker::WWW::Robot::RobotUriQueue;

use strict;
use URI;
#use Tie::Array::Sorted;
use Tie::IxHash;

sub new {
  my $class = shift;

  my $self = bless {}, $class;
  #tie @{$self->{open_time_list}},
  #  "Tie::Array::Sorted", sub { $_[0]->[0] <=> $_[1]->[0] };
  $self->{open_time_list} = [];

  return $self;
}

sub next_open_time {
  my $self = shift;

  if (scalar @{$self->{open_time_list}} == 0) {
    return undef;
  }
  return $self->{open_time_list}[0]->[0];
}

sub next_uri {
  my $self = shift;

  if (scalar @{$self->{open_time_list}} == 0) {
    return undef;
  }

  my $host = $self->{open_time_list}[0]->[1];

  my $uri_list_size = (tied %{$self->{host}{$host}{uri}})->Length;
  die if $uri_list_size eq 0;
  my ($uri, $count) = (tied %{$self->{host}{$host}{uri}})->Shift;
  if ($uri_list_size == 1) {
    shift(@{$self->{open_time_list}});
    delete $self->{host}{$host};
  }
  return $uri;
}

sub host_count {
  my $self = shift;

  return scalar @{$self->{open_time_list}};
}

sub uri_open_time {
  my ($self, $uri) = @_;

  my $uri_obj = URI->new($uri);
  my $host = $uri_obj->host;

  if (exists $self->{host}{$host}) {
    return $self->{host}{$host}{open_time};
  }
  return undef;
}

sub add {
  my ($self, $uri, $open_time) = @_;

  unless (defined $open_time) {
    $open_time = 0;
  }

  my $uri_obj = URI->new($uri);
  my $host = $uri_obj->host;
  if (exists $self->{host}{$host}) {
    $self->{host}{$host}{uri}{$uri}++;
    if ($self->{host}{$host}{open_time} < $open_time) {
      $self->_remove_from_open_time_list($self->{host}{$host}{open_time},
                                         $host);
      $self->{host}{$host}{open_time} = $open_time;
      $self->_add_to_open_time_list($open_time, $host);
    }
  }
  else {
    tie %{$self->{host}{$host}{uri}}, "Tie::IxHash";
    $self->{host}{$host}{uri}{$uri}++;
    $self->{host}{$host}{open_time} = $open_time;
    $self->_add_to_open_time_list($open_time, $host);
  }
}

sub print_queue {
  my ($self, $handle) = @_;

  for my $host (keys %{$self->{host}}) {
    for my $uri (keys %{$self->{host}{$host}{uri}}) {
      print $handle $uri, "\n";
    }
  }
}

sub _remove_from_open_time_list {
  my ($self, $open_time, $host) = @_;

  my $pos = $self->_search($host, $open_time);
  return unless defined($pos);
  splice(@{$self->{open_time_list}}, $pos, 1);
}

sub _add_to_open_time_list {
  my ($self, $open_time, $host) = @_;

  #push(@{$self->{open_time_list}}, [$open_time, $host]);

  my ($lo, $hi) = (0, scalar @{$self->{open_time_list}} - 1);
  while ($hi >= $lo) {
    my $mid = int(($lo+$hi)/2);
    if ($self->{open_time_list}[$mid]->[0] == $open_time) {
      splice(@{$self->{open_time_list}}, $mid, 0, [$open_time, $host]);
      return;
    }
    elsif ($self->{open_time_list}[$mid]->[0] < $open_time) {
      $lo = $mid + 1;
    }
    else {
      $hi = $mid - 1;
    }
  }
  splice(@{$self->{open_time_list}}, $lo, 0, [$open_time, $host]);
}

sub _search {
  my ($self, $host, $open_time) = @_;

  my ($lo, $hi) = (0, scalar @{$self->{open_time_list}} - 1);
  while ($hi >= $lo) {
    my $mid = int(($lo+$hi)/2);
    if ($self->{open_time_list}[$mid]->[0] == $open_time) {
      if ($self->{open_time_list}[$mid]->[1] eq $host) {
        return $mid;
      }
      my $mid_prev = $mid - 1;
      while ($mid_prev >= 0) {
        last if $self->{open_time_list}[$mid_prev] != $open_time;
        if ($self->{open_time_list}[$mid_prev]->[1] eq $host) {
          return $mid_prev;
        }
      }
      my $mid_next = $mid + 1;
      while ($mid_next < $#{$self->{open_time_list}}) {
        last if $self->{open_time_list}[$mid_next] != $open_time;
        if ($self->{open_time_list}[$mid_next]->[1] eq $host) {
          return $mid_next;
        }
      }
      return undef;
    }
    elsif ($self->{open_time_list}[$mid]->[0] < $open_time) {
      $lo = $mid + 1;
    }
    else {
      $hi = $mid - 1;
    }
  }
  return undef;
}

1;

==> ja-WiKicker-0.26.tbz: lib/perl5/site_perl/5.8.9/WiKicker/WWW/Robot/RobotUriQueue.pm <==
#
# WiKicker
#
#  RobotUriQueue.pm
#
#
# Copyright (C) 2003 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: RobotUriQueue.pm,v 1.2 2004/08/03 12:55:52 naney Exp $
#
#

package WiKicker::WWW::Robot::RobotUriQueue;

use strict;
use URI;
#use Tie::Array::Sorted;
use Tie::IxHash;

sub new {
  my $class = shift;

  my $self = bless {}, $class;
  #tie @{$self->{open_time_list}},
  #  "Tie::Array::Sorted", sub { $_[0]->[0] <=> $_[1]->[0] };
  $self->{open_time_list} = [];

  return $self;
}

sub next_open_time {
  my $self = shift;

  if (scalar @{$self->{open_time_list}} == 0) {
    return undef;
  }
  return $self->{open_time_list}[0]->[0];
}

sub next_uri {
  my $self = shift;

  if (scalar @{$self->{open_time_list}} == 0) {
    return undef;
  }

  my $host = $self->{open_time_list}[0]->[1];

  my $uri_list_size = (tied %{$self->{host}{$host}{uri}})->Length;
  die if $uri_list_size eq 0;
  my ($uri, $count) = (tied %{$self->{host}{$host}{uri}})->Shift;
  if ($uri_list_size == 1) {
    shift(@{$self->{open_time_list}});
    delete $self->{host}{$host};
  }
  return $uri;
}

sub host_count {
  my $self = shift;

  return scalar @{$self->{open_time_list}};
}

sub uri_open_time {
  my ($self, $uri) = @_;

  my $uri_obj = URI->new($uri);
  my $host = $uri_obj->host;

  if (exists $self->{host}{$host}) {
    return $self->{host}{$host}{open_time};
  }
  return undef;
}

sub add {
  my ($self, $uri, $open_time) = @_;

  unless (defined $open_time) {
    $open_time = 0;
  }

  my $uri_obj = URI->new($uri);
  my $host = $uri_obj->host;
  if (exists $self->{host}{$host}) {
    $self->{host}{$host}{uri}{$uri}++;
    if ($self->{host}{$host}{open_time} < $open_time) {
      $self->_remove_from_open_time_list($self->{host}{$host}{open_time},
                                         $host);
      $self->{host}{$host}{open_time} = $open_time;
      $self->_add_to_open_time_list($open_time, $host);
    }
  }
  else {
    tie %{$self->{host}{$host}{uri}}, 'Tie::IxHash';
    $self->{host}{$host}{uri}{$uri}++;
    $self->{host}{$host}{open_time} = $open_time;
    $self->_add_to_open_time_list($open_time, $host);
  }
}

sub print_queue {
  my ($self, $handle) = @_;

  for my $host (keys %{$self->{host}}) {
    for my $uri (keys %{$self->{host}{$host}{uri}}) {
      print $handle $uri, "\n";
    }
  }
}

sub _remove_from_open_time_list {
  my ($self, $open_time, $host) = @_;

  my $pos = $self->_search($host, $open_time);
  return unless defined($pos);
  splice(@{$self->{open_time_list}}, $pos, 1);
}

sub _add_to_open_time_list {
  my ($self, $open_time, $host) = @_;

  #push(@{$self->{open_time_list}}, [$open_time, $host]);

  my ($lo, $hi) = (0, scalar @{$self->{open_time_list}} - 1);
  while ($hi >= $lo) {
    my $mid = int(($lo+$hi)/2);
    if ($self->{open_time_list}[$mid]->[0] == $open_time) {
      splice(@{$self->{open_time_list}}, $mid, 0, [$open_time, $host]);
      return;
    }
    elsif ($self->{open_time_list}[$mid]->[0] < $open_time) {
      $lo = $mid + 1;
    }
    else {
      $hi = $mid - 1;
    }
  }
  splice(@{$self->{open_time_list}}, $lo, 0, [$open_time, $host]);
}

sub _search {
  my ($self, $host, $open_time) = @_;

  my ($lo, $hi) = (0, scalar @{$self->{open_time_list}} - 1);
  while ($hi >= $lo) {
    my $mid = int(($lo+$hi)/2);
    if ($self->{open_time_list}[$mid]->[0] == $open_time) {
      if ($self->{open_time_list}[$mid]->[1] eq $host) {
        return $mid;
      }
      my $mid_prev = $mid - 1;
      while ($mid_prev >= 0) {
        last if $self->{open_time_list}[$mid_prev] != $open_time;
        if ($self->{open_time_list}[$mid_prev]->[1] eq $host) {
          return $mid_prev;
        }
      }
      my $mid_next = $mid + 1;
      while ($mid_next < $#{$self->{open_time_list}}) {
        last if $self->{open_time_list}[$mid_next] != $open_time;
        if ($self->{open_time_list}[$mid_next]->[1] eq $host) {
          return $mid_next;
        }
      }
      return undef;
    }
    elsif ($self->{open_time_list}[$mid]->[0] < $open_time) {
      $lo = $mid + 1;
    }
    else {
      $hi = $mid - 1;
    }
  }
  return undef;
}

1;
